#pragma once

#include "stdafx.h"

class Interpreter
{
public:

	Interpreter();
	~Interpreter();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    clearParamsInput
	*Description: clear the paramsInput before filled in with the new data
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void clearParamsInput();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    checkCommentLine
	*Description: check if the streamed line is a comment line or empty
	*Parameters:  std::string command: input line
	*Returns:     bool true: it is a comment line or empty line
	*             bool false: it is not a comment line or empty line
	*******************************************************************************/
	bool checkCommentLine(std::string command);

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    checkModeParameters
	*Description: set the family, protocl, baud rate value, and max buffer size
	*             based on MODE command from the user
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void checkModeParameters();
	

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    interpretCommandParams
	*Description: validate number of parameters given in the command
	*Parameters:  string command : input command line from the user
	*Returns:	  none
	*******************************************************************************/
	void interpretCommandParams(string command);

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    matchCommand
	*Description: match input command with the available commands
	*             and check the validity of number of parameters
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void matchCommand();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    retActualCommandParams
	*Description: return the actualCommandParams to be accessed by the other classes
	*Parameters:  none
	*Returns:     ActualCommandParams actualCommandParams
	*******************************************************************************/
	ActualCommandParams retActualCommandParams();
	
	/*******************************************************************************
	*Class:       Interpreter
	*Function:    setupNewBaudRate
	*Description: set the new baud rate value into actualCommandParamsStruct
	*Parameters:  uint32_t newBaudRate: new baud rate value
	*Returns:     none
	*******************************************************************************/
	void setupNewBaudRate(uint32_t newBaudRate);
 	

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    retResponse
	*Description: switch case to interpret the response based on BSL command
	*Parameters:  uint8_t *buf: pointer to the response buffer
	*Returns:     std::string temp: string response to be shown for the user
	*******************************************************************************/
	std::string retResponse(uint8_t *buf, uint8_t *headerBuf);

	/*******************************************************************************
	*Class:       Interpreter
	*Function:	  isRxDataPassInputCommand
	*Description: check that executed command  is RX_xxx command with 
	*             input data file parameter
	*Parameters:  none
	*Returns:	  (true) : command executed is RX command with input
	*                      data file parameter
	*             (false) : command  xecuted is not RX command with input
	*                      data file parameter
	*******************************************************************************/
	bool isRxDataPassInputCommand();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:	  isRxPassword
	*Description: check that executed command  is RX_PASSWORD
	*Parameters:  none
	*Returns:	  (true) : command  executed is RX_PASSWORD
	*             (false) : command  executed is not RX_PASSWORD
	*******************************************************************************/
	bool isRxPassword();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:	  isRxPassDefaultCommand
	*Description: check that executed command  is RX_PASSWORD using default password
	*Parameters:  none
	*Returns:	  (true) : command  executed is RX_PASSWORD using default
	*                      password
	*             (false) : command  executed is not RX_PASSWORD using default
	*                      password
	*******************************************************************************/
	bool isRxPassDefaultCommand();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:	  isTxDataCommand
	*Description: check that executed command  is TX_DATA_BLOCK
	*Parameters:  none
	*Returns:	  (true) : command  executed is TX_DATA_BLOCK
	*             (false) : command  executed is not TX_DATA_BLOCK
	*******************************************************************************/
	bool isTxDataCommand();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:	  isCommandReqNoResponse
	*Description: check that executed command  is DELAY or VERBOSE
	*             these commands do not require any transmission 
	*             to the target device
	*Parameters:  none
	*Returns:	  (true) : command  executed is DELAY or VERBOSE
	*             (false) : command  executed is DELAY or VERBOSE
	*******************************************************************************/
	bool isCommandReqNoResponse();

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    retCrcBsl
	*Description: return the crc value from the response
	*Parameters:  none
	*Returns:     uint16_t crcBsl: crc from BSL device
	*******************************************************************************/
	uint16_t retCrcBsl();

private:

	//Response byte from
	const uint8_t HEADER_BYTE = 0x80;
	const uint8_t USB_HEADER_BYTE = 0x3F;
	const uint8_t BSL_DATA_REPLY = 0x3A;
	const uint8_t BSL_MESSAGE_REPLY = 0x3B;
	const uint8_t FLASH_ACK = 0x00;

	// Error Codes from the API
	const uint8_t SUCCESSFUL_OPERATION = 0x00;
	const uint8_t NO_EXCEPTIONS = 0x00;
	const uint8_t FLASH_WRITE_CHECK_FAILED = 0x01;
	const uint8_t FLASH_FAIL_BIT_SET = 0x02;
	const uint8_t VOLTAGE_CHANGE_DURING_PROGRAM = 0x03;
	const uint8_t BSL_LOCKED = 0x04;
	const uint8_t BSL_PASSWORD_ERROR = 0x05;
	const uint8_t BYTE_WRITE_FORBIDDEN = 0x06;
	const uint8_t BYTE_UNKNOWN_COMMAND = 0x07;

	ActualCommandParams actualCommandParams;
	
	uint16_t  crcBsl;
	uint16_t  bufferSize;
	uint8_t   bslVersion[5];
	uint16_t  bslVersion32[5];
	bool resetComm;

	/*******************************************************************************
	*Class:       Interpreter
	*Function:    retCommandIdx
	*Description: get the Command identity
	*Parameters:  uint8_t idx
	*Returns:     Command(enumclass) identity
	*******************************************************************************/
	Command retCommandIdx(uint16_t idx);

	/*******************************************************************************
	*Class:       Interpreter
	*Function:	  checkBaudRate
	*Description: check the input baud rate from the user based on the protocol 
	*             chosen
	*Parameters:  Protocol protocol : communication protocol
	*             uint32_t baudRate	: baud rate value
	*Returns:	  (true) : protocol and baud rate value valid
	*******************************************************************************/
	bool checkBaudRate(Protocol protocol, uint32_t baudRate);

	/*******************************************************************************
	*Class:       Interpreter
	*Function:	  checkHeader
	*Description: interpret the received response to string message
	*Parameters:  uint8_t *bufHeader    : received ack byte 
	* 		      uint8_t* messageReply	: received message reply
	*             uint8_t *coreResponse	: received core response
	*Returns:	  std::string : response
	*******************************************************************************/
	std::string checkHeader(uint8_t *bufHeader, uint8_t* messageReply, uint8_t *coreResponse);
	
	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    intToHex
	*Description: convert the uint8_t type into hex in string type
	*             need to cast the uint8_t to uint32_t
	*Parameters:  uint8_t input: integer input to be converted to hex
	*Returns:     std::string  stream.str(): the string format of hex value
	*******************************************************************************/
	std::string intToHex(uint8_t input);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    intToHexCrc
	*Description: convert the uint32_t type into hex in string type
	*Parameters:  uint8_t input: integer input to be converted to hex
	*Returns:     std::string  stream.str(): the string format of hex value
	*******************************************************************************/
	std::string intToHexCrc(uint16_t input);

	/*******************************************************************************
	*Class:       ScripterStream
	*Function:    intToString
	*Description: convert the uint8_t type into hex in string type with '0x' prefix
	*Parameters:  uint8_t input: integer input to be converted to hex
	*Returns:     std::string  stream.str(): the string format of hex value
	*******************************************************************************/
	std::string intToString(uint8_t input);

	// The functions below are functions to interpret target device response 
	// specific for each command
	// The parameters: 
	// uint8_t *buf : pointer to the rxBuffer where the message and core 
	//                responses are stored
	// uint8_t *ackByte : pointer to the header byte buffer where the ack
	//                    byte is stored
 	std::string changeBaudRate(uint8_t *buf, uint8_t *ackByte);
	std::string crcCheck32(uint8_t *buf, uint8_t *ackByte);
	std::string crcCheck(uint8_t *buf, uint8_t *ackByte);
	std::string eraseSegment32(uint8_t *buf, uint8_t *ackByte);
	std::string eraseSegment(uint8_t *buf, uint8_t *ackByte);
	std::string setPC(uint8_t *buf, uint8_t *ackByte);
	std::string massErase(uint8_t *buf, uint8_t *ackByte);
	std::string rxDataBlockFast(uint8_t *buf, uint8_t *ackByte);
	std::string rxDataBlock(uint8_t *buf, uint8_t *ackByte);
	std::string rxDataBlock32(uint8_t *buf, uint8_t *ackByte);
	std::string rxPassword32(uint8_t *buf, uint8_t *ackByte);
	std::string rxPassword(uint8_t *buf, uint8_t *ackByte);
	std::string toggleInfo(uint8_t *buf, uint8_t *ackByte);	 
	std::string rebootReset(uint8_t *buf, uint8_t *ackByte);
	std::string txDataBlock32(uint8_t *buf, uint8_t *ackByte);
	std::string txDataBlock(uint8_t *buf, uint8_t *ackByte);
	std::string txBslVersion32(uint8_t *buf, uint8_t *ackByte);
	std::string txBslVersion(uint8_t *buf, uint8_t *ackByte);
	std::string txBufferSize(uint8_t *buf, uint8_t *ackByte);
	std::string unknownCommand();
	

};


