#pragma once

#include "stdafx.h"

class UartComm : public CommInterface
{
public:

	UartComm();
	~UartComm();

	/*******************************************************************************
	*Class:       UartComm
	*Function:	  init
	*Description: Setup the initialization of uart communication
	*Parameters:  ActualCommandParams actualCommandParams
	*Returns:	  none
	*******************************************************************************/
	void init(ActualCommandParams actualCommandParams) override;
	
	/***********************************************************************************
	*Class:       UartComm
	*Function:    transmitBufferInit432
	*Description: do the invoke for MSP432 by sending 0xFF until the device answering
	*Parameters:  none
	*Returns:     none
	***********************************************************************************/
	void transmitBufferInit432() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:    transmitBuffer
	*Description: send the BSL command
	*Parameters:  DataBuffer dataBuffer : BSL command that is going to be sent
	*Returns:     none
	***********************************************************************************/
	 void transmitBuffer(DataBuffer dataBuffer) override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  receiveBuffer
	*Description: receive byte(s) response from target device
	*Parameters:  int32_t size : number of bytes that should be received by the
	*                            scripter
	*             (-1) means only one byte response without wrapped in BSL data packet
	*             (0) means no byte response
	*			  (n>0) means n bytes wrapped in BSL data packet
	*Returns:     uint8_t* : pointer to the receive buffer
	***********************************************************************************/
	uint8_t* receiveBuffer(int32_t size) override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  close()
	*Description: close port communication
	*Parameters:  none
	*Returns:     none
	***********************************************************************************/
	void close() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  changeBaudRate
	*Description: change baud rate value
	*Parameters:  uint32_t baudRate : new baud rate value
	*Returns:     none
	***********************************************************************************/
	void changeBaudRate(uint32_t baudRate) override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retBytesWritten()
	*Description: return number of bytes including the command byte
	*Parameters:  none
	*Returns:     uint32_t : number of bytes written to the device
	***********************************************************************************/
	uint32_t retBytesWritten() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retBytesRead()
	*Description: return number of bytes (pure data without any command byte, check sum,
	*             or header bytes)
	*Parameters:  none
	*Returns:     uint32_t : number of bytes read from the device
	***********************************************************************************/
	uint32_t retBytesRead() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retBytesPacketRead()
	*Description: return number of bytes included the command byte
	*Parameters:  none
	*Returns:     uint32_t : number of bytes read from the device
	***********************************************************************************/
	uint32_t retBytesPacketRead() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retHeaderBuffer()
	*Description: return pointer of header buffer
	*Parameters:  none
	*Returns:     uint8_t* : pointer to the header buffer
	***********************************************************************************/
	uint8_t* retHeaderBuffer() override;

private:

	boost::asio::io_service io_service;
	boost::asio::serial_port port;
	boost::asio::deadline_timer timer;

	uint8_t  rxBuffer[UART_RX_BUFFER_SIZE];
	uint8_t  headerBuffer[UART_HEADER_BUFFER_SIZE];

	uint32_t actualBaudRate;
 	uint16_t crc; 
	
	void cleanRxBuffer();
	void cleanHeaderBuffer();
	uint8_t getCheckSumLow();
	uint8_t getCheckSumHigh();
	void sendByte(uint8_t *data);
	void checkSumCalculate(uint8_t data);

	bool timerEvent;
	bool readEvent;
	bool cancelled;
  	bool waitAck;
	bool waitHeader;
	bool waitData;
	uint32_t actSize;
	uint32_t expSize;
	int32_t pureData = 0;
	int32_t numData;
	uint32_t bytesReceived;

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  fsmReceiveBuffer
	*Description: control the receiving data of uart protocol
	*Parameters:  int32_t size: number o f bytes should be received
	*Returns:	  (true) : process is valid
	*             (false) : process is invalid
	***********************************************************************************/
	bool fsmReceiveBuffer(int32_t size);

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  resetIdxRead
	*Description: reset all indexes that are used in the receiving process
	*Parameters:  none
	*Returns:	  none
	***********************************************************************************/
	void resetIdxRead();

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  setTimer
	*Description: set deadline timer with delay input in the parameter
	*Parameters:  noen
	*Returns:	  uint32_t duration : delay in millisecond
	***********************************************************************************/
	void setTimer(uint32_t duration);

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  onTimer
	*Description: set deadline timer on
	*Parameters:  const boost::system::error_code& ec : error message
	*Returns:	  none
	***********************************************************************************/
	void onTimer(const boost::system::error_code& ec);

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  startRead
	*Description: set deadline timer on
	*Parameters:  const boost::system::error_code& ec : error message
	*Returns:	  uint8_t* buf : pointer to the buffer where data will be stored
	*             size_t numBytes : number of bytes should be received
	***********************************************************************************/
 	void startRead(uint8_t* buf, size_t numBytes);//void startRead(size_t offset, size_t numBytes);

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  onRead
	*Description: set read process on
	*Parameters:  const boost::system::error_code& ec : error message
	*             size_t numBytes : number of received bytes
	*Returns:	  none
	***********************************************************************************/
	void onRead(const boost::system::error_code& ec, size_t numBytes);
  	
	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  cleanup
	*Description: close port
	*Parameters:  none
	*Returns:	  none
	***********************************************************************************/
	void cleanup();

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  isOpen
	*Description: check status of comm port
	*Parameters:  none
	*Returns:	  (true) port is accessible
	*             (false) port is not accessible
	***********************************************************************************/
	bool isOpen() const;

	/***********************************************************************************
	*Class:		  UartComm
	*Function:	  cancel
	*Description: cancel respective process (async_read or timer) when certain condition
	*             obtained
	*Parameters:  none
	*Returns:	  none
	***********************************************************************************/
	void cancel();
};


